/*
 *
 *  Created on: 04 March 2014
 *      Author: Anthony
 */
#include "costFunction.h"

#define DEBUG		0
#define DEBUGCONFLICT	0
//#define EPSILON 	0.000001
#define EPSILON 	0.001
#define CHECKLISTSIZE	2000

float* coeffList[200];	//Ptr on the coeff for the valid links 
int nbOfValidLinks;	//Size of coeffList

//Check if the route routeNumber is active between id1 and id2
//i.e. if the routeNumberth entry of the routing table of node id1 is not empty and has id2 as next hop.
//Moreover, check if freq is used between these two nodes.
//Return 1 if yes, 0 otherwise.
int routeActiveOnFreq(int id1, int id2, int freq, int routeNumber)
{
	if(nodes[id1].routingTable[routeNumber].nextHop!=id2) return(0);
	if(linkUsedFrequency(id1,id2,freq)!=1) return(0);

	return(1);
}

//If id1 and id2 use freq, update the coeffArray for each route with id2 as next hop. 
//Update means add 1/getMetric() to the current coeff. 
void updateCoefficientArray(float* coeffArray, int id1, int id2, int freq, int nbOfRoutes)
{
	int i;

	if(nbOfRoutes>MAX_NBOFROUTES)
	{
		fprintf(stderr,"Error in updateCoeffcientArray() in costFunction.c: MAX_NBOFROUTES (%d) < nbOfRoutes (%d)\n",MAX_NBOFROUTES,nbOfRoutes);
		exit(1);
	}

	for (i=0;i<nbOfRoutes;i++)
	{
		//We check in both direction (route are directional)
		if(routeActiveOnFreq(id1,id2,freq,i) || routeActiveOnFreq(id2,id1,freq,i))
		{ 
			coeffArray[i]=coeffArray[i]+1.0/(1.0*getMetric(id1,id2));
			if(DEBUG) fprintf(stderr,"Add 1.0/%f for link (%d,%d) on pathNumber %d\n",getMetric(id1,id2),id1,id2,i);
		}
	}
}

//Compute the coefficient of the inequality (for the inequality linear systems)
//Must be done for each link on the paths.
//These coeffcients are added to an array associate to the corresponding routing entry on node id1.
//So we assume that there is a path/route to destination between id1 and id2 
//The frequency and radio MUST be updated

void setCoefficientLink(int id1, int id2, int pathNumber, int freq, int nbOfRoutes)
{
    	int i, j, idN1, idN2, idN3;
	float coefficient[MAX_NBOFROUTES];//NBOFROUTES is set in type.h

	int* checkList[2];//Keep all links already checked/counted in the conflict graph
    	int checkListNumber=0;

    	if( (checkList[0]=(int *) malloc(CHECKLISTSIZE*sizeof(int)))==NULL) {perror("Error malloc() in conflictGraphLink2()"); exit(2);}
    	if( (checkList[1]=(int *) malloc(CHECKLISTSIZE*sizeof(int)))==NULL) {perror("Error malloc() in conflictGraphLink2()"); exit(2);}


	for(i=0;i<MAX_NBOFROUTES;i++) coefficient[i]=0.0;

	if(DEBUG) fprintf(stderr,"DEBUG FUNCTION setCoeffcientLink(%d,%d) on Freq=%d\n",id1,id2,freq);

    	//Traitement auprès du noeud id1
	for(i=0;i<nodes[id1].nbN1;i++)
	{
		idN1=nodes[id1].N1[i].id;//index of the neighbor - may be id2

		//Direct neighbor
		checkListNumber=updateCheckList(checkList, id1, idN1, checkListNumber);
		updateCoefficientArray(coefficient,id1, idN1, freq, nbOfRoutes);
		if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-1\n",id1,idN1);

		//2 1-neighbors together
		for(j=i+1;j<nodes[id1].nbN1;j++)
		{
			checkListNumber=updateCheckList(checkList, idN1, nodes[id1].N1[j].id, checkListNumber);
			updateCoefficientArray(coefficient,idN1,nodes[id1].N1[j].id, freq, nbOfRoutes);
			if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-2\n",idN1,nodes[id1].N1[j].id);
		}

		//neighbor with 2-neighbors
		for(j=0;j<nodes[id1].nbN2;j++)
		{
			checkListNumber=updateCheckList(checkList, idN1, nodes[id1].N2[j].id, checkListNumber);
			updateCoefficientArray(coefficient,idN1,nodes[id1].N2[j].id, freq, nbOfRoutes);
			if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-3\n",idN1,nodes[id1].N2[j].id);
		}

	}

	//2 2-neighbors together
	for(i=0;i<nodes[id1].nbN2;i++)
	{
		idN2 = nodes[id1].N2[i].id;//index of the neighbor
		for(j=i+1;j<nodes[id1].nbN2;j++)
		{
			checkListNumber=updateCheckList(checkList, idN2, nodes[id1].N2[j].id, checkListNumber);
			updateCoefficientArray(coefficient,idN2,nodes[id1].N2[j].id, freq, nbOfRoutes);
			if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-4\n",idN2,nodes[id1].N2[j].id);
		}
	}

	//neighbor with 3-neighbors of node id1
	for(i=0;i<nodes[id1].nbN2;i++)
	{
		idN2 = nodes[id1].N2[i].id;
		for(j = 0 ; j < nodes[idN2].nbN1;j++)
		{
			idN3 = nodes[idN2].N1[j].id;
			//DEBUG DEBUG
			if( is3Neighbor(idN3,id1))
			{ 
				updateCoefficientArray(coefficient,idN2,idN3, freq, nbOfRoutes);
				checkListNumber=updateCheckList(checkList, idN2, idN3, checkListNumber);
			}
			if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-5\n",idN2,idN3);
		}
	}


	//  Traitement auprès du noeud id2

	//2 2-neighbors together
	for(i=0;i<nodes[id2].nbN2;i++)
	{
		idN2 = nodes[id2].N2[i].id;//index of the neighbor
		for(j=i+1;j<nodes[id2].nbN2;j++)
		{
			if(isIntheCheckedList(checkList, idN2,nodes[id2].N2[j].id, checkListNumber)==0)
			{
				checkListNumber=updateCheckList(checkList,idN2,nodes[id2].N2[j].id, checkListNumber);
				updateCoefficientArray(coefficient,idN2,nodes[id2].N2[j].id, freq, nbOfRoutes);
			}
			if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-6\n",idN2,nodes[id2].N2[j].id);
		}
	}

	// 3-neighbors of node id2
	for(i = 0 ; i < nodes[id2].nbN2;i++)
	{
		idN2 = nodes[id2].N2[i].id;
		for(j = 0 ; j < nodes[idN2].nbN1;j++)
		{
			idN3 = nodes[idN2].N1[j].id;

			// idN3 est le voisinage 3-sauts de id2
			if( (isIntheCheckedList(checkList, idN2,idN3, checkListNumber)==0) && (is3Neighbor(idN3,id2)==1) && (isNeighbor(id1,idN3)==0) && (is2Neighbor(id1,idN3)==0) && (is3Neighbor(id1,idN3)==0))  // et idN3 n'est pas de voisinage 1,2,3 saut de id1
			{
				checkListNumber=updateCheckList(checkList,idN2,idN3,checkListNumber);
				updateCoefficientArray(coefficient,idN2,idN3, freq, nbOfRoutes);
				if(DEBUGCONFLICT) printf("link (%d,%d) in conflict-7\n",idN2,idN3);
			}
		}
	}

	//We copy the coefficients in the routing table of node id1
	for(i=0;i<nbOfRoutes;i++) nodes[id1].routingTable[pathNumber].coeff[i]=coefficient[i];
	


	if(DEBUG)
	{ 
		fprintf(stderr,"Coefficient = [");
		for(i=0;i<nbOfRoutes;i++) fprintf(stderr,"%f ",coefficient[i]);
		fprintf(stderr,"]\n");
	}

	free(checkList[0]); free(checkList[1]);
}

//Set all coefficient at 0 from node start on pathNumber
void setCoefficientNull(int start, int pathNumber)
{
	int current, next, destination;

	current=start;
	destination=nbOfNodes-1; 

	while(current!=destination)
	{
		next=nodes[current].routingTable[pathNumber].nextHop;
		nodes[current].routingTable[pathNumber].coeff[pathNumber]=0.0;
		current=next;
	}

}//end of fct setCoeffcientNull()



//algo=0 if we search for the optimal
//algo=1 if we use the heuristic
//The difference is that for the optimal all the links of the path are 0 if there is a link that is not valid (no common frequency between neighbor)
//whereas we set the coefficient until the no valid link in the case of the heuristic.
//Return the number of valid paths even partial (set the size of the linear system to solve).
int setCoefficientAllNodes(int nbOfRoutes, int algo)
{
	int pathNumber, validPathNb=0, quitWhile;
	int src, next, destination, indexN1, freq, nbOfValidLinkPath;

	destination=nbOfNodes-1; 	
	nbOfValidLinks=0;	//Global variable

	//We run through all paths (for active path)
	for(pathNumber=0;pathNumber<nbOfRoutes;pathNumber++)
	{
		src=0; 
		//la condition ci-dessous n'arrive que si le chemin a ete explicitement invalide par l'algorithme d'assignation. 
		if(nodes[0].routingTable[pathNumber].destination==-1)
		{
			//The route is not assigned at all: set coefficient to 0
			//setCoefficientNull(src,pathNumber);	
			nodes[src].routingTable[pathNumber].coeff[pathNumber]=0.0;
			continue;
		}

		//Active route - run through path pathNumber
		quitWhile=0; nbOfValidLinkPath=0;
		while( (src!=destination) && (quitWhile==0) )
		{
			next=nodes[src].routingTable[pathNumber].nextHop;
			if(linkExist2(src,next)==0)//Path broken 
			{
				//fprintf(stderr,"--DEBUG-- Path %d on link (%d,%d) is broken\n",pathNumber,src,next);
				switch(algo)
				{
					case 0: setCoefficientNull(0, pathNumber);//The whole path is emptied
						quitWhile=1;//We pass to the next path
						break;
					case 1: setCoefficientNull(src, pathNumber);//The path from this node is emptied
						quitWhile=1;//We pass to the next path
						break;
					case 2: src=next;//increment
						break;
					default: fprintf(stderr,"Error setCoefficientAllNodes(): value of the algo is not valid, it should be 0, 1 or 2 (it is %d)\n",algo);
						exit(1);
				}
			} else {
				indexN1=getIndexN1(nodes[src],next);//Search freq used between src and next
				freq=nodes[src].N1[indexN1].frequency;
				setCoefficientLink(src, next, pathNumber, freq, nbOfRoutes);
				coeffList[nbOfValidLinks]=nodes[src].routingTable[pathNumber].coeff;//Is a Ptr (coeff is an array) on the coeffs for this link (src->next) 
				nbOfValidLinks++;//Size of coeffList
				nbOfValidLinkPath++;//increment the number of valid link on this path
			
				src=next;
			}
		}

		if( (algo==0) && (src==destination))
		{
			validPathNb++;	//Count only full path as valid

		}
		if( (algo==1) && (src!=0)) validPathNb++;		//Count partial path as valid (with at least one link)
		if( (algo==2) && (nbOfValidLinkPath>0)) validPathNb++;	//Consider a path valid if there is at least one correct assignment on this path
	}//end of for on pathNumber

	return(validPathNb);
}

/*
* coeffEq1=[coeff1, coeff2] for equation: coef1*x+coef2*y=1 (idem coeffEq2)
* result will host the results (x,y)
* Return the sum x+y if there is a solution ; return -1.0 otherwise. 
* index1, index2 are the coefficient to take into account (it is not always 0 and 1 / it may be 0 and 2) ; it depends on the valid routes
*/
float solve2equations(float* coeffEq1, float* coeffEq2, int index1, int index2, float* result)
{
	float determinant;

	determinant=coeffEq2[index1]*coeffEq1[index2]-coeffEq1[index1]*coeffEq2[index2];//It is -determinant but it is more convenient (for the solution) and as we compare to 0...
	result[0]=0.0; result[1]=0.0;

	if( (determinant==0.0) || fabsf(determinant<EPSILON))
	{
		//Degenerate cases: 
		//	case1: coeffEq1[0]==0 and coeffEq2[0]==0 
		//	case2: coeffEq1[1]==0 and coeffEq2[1]==0
		//	case3: coeffEq1[0]==0 and coeffEq2[1]==0
		//	case4: coeffEq1[1]==0 and coeffEq2[0]==0
		//	case5: equations are dependent --> in this case take the one with greatest coefficient

		//Case 1
		if(coeffEq1[index1]<EPSILON && coeffEq2[index1]<EPSILON)
		{
			result[1]=1.0/MAX(coeffEq1[index2],coeffEq2[index2]);
			return(result[0]+result[1]);
		}

		//Case 2
		if(coeffEq1[index2]<EPSILON && coeffEq2[index2]<EPSILON)
		{
			result[0]=1.0/MAX(coeffEq1[index1],coeffEq2[index1]);
			return(result[0]+result[1]);
		}

		//Case 3
		if(coeffEq1[index1]<EPSILON && coeffEq2[index2]<EPSILON)
		{
			result[0]=1.0/coeffEq2[index1];
			result[1]=1.0/coeffEq1[index2];
			return(result[0]+result[1]);
		}

		//Case 4
		if(coeffEq1[index2]<EPSILON && coeffEq2[index1]<EPSILON)
		{
			result[0]=1.0/coeffEq1[index1];
			result[1]=1.0/coeffEq2[index2];
			return(result[0]+result[1]);
		}

		//Case 5 
		if(coeffEq1[index1]>coeffEq2[index1])//The first equation "is more constrained"
		{
			//In this case the optimization consists in taking 0 for one of the two paths (this solution optimizes the sum)
			//if(coeffEq1[index1]>coeffEq1[index2]) result[1]=1.0/coeffEq1[index2];
			//else result[1]=1.0/coeffEq1[index1];

			if(MIN(coeffEq1[index1],coeffEq1[index2])>0.0) result[1]=1.0/MIN(coeffEq1[index1],coeffEq1[index2]);
			else result[1]=1.0/MAX(coeffEq1[index1],coeffEq1[index2]);

			//If there is equality we share
			if(fabs(coeffEq1[index1]-coeffEq1[index2]) < EPSILON){ result[1]=1.0/(2.0*coeffEq1[index2]); result[0]=1.0/(2.0*coeffEq1[index2]);}

		} else { //The 2nd equation is more constrained
			//if(coeffEq2[index1]>coeffEq2[index2]) result[1]=1.0/coeffEq2[index2];
			//else result[1]=1.0/coeffEq2[index1];

			if(MIN(coeffEq2[index1],coeffEq2[index2])>0.0) result[1]=1.0/MIN(coeffEq2[index1],coeffEq2[index2]);
			else result[1]=1.0/MAX(coeffEq2[index1],coeffEq2[index2]);

			//If there is equality we share
			if(fabs(coeffEq2[index1]-coeffEq2[index2]) < EPSILON){ result[1]=1.0/(2.0*coeffEq2[index2]); result[0]=1.0/(2.0*coeffEq2[index2]);}
		}

		return(result[0]+result[1]);
	}

	//There is a solution
	result[0]=(coeffEq1[index2]-coeffEq2[index2])/determinant;
	result[1]=(coeffEq2[index1]-coeffEq1[index1])/determinant;

	//Results may be negative
	if(result[0]<0.0) result[0]=0.0;
	if(result[1]<0.0) result[1]=0.0;

	return(result[0]+result[1]);
}//end of fct solve2equations


//coeffEq1=[coeff1, coeff2, coeff3] for equation: coeff1*x+coeff2*y+coeff3*z=1 (idem coeffEq2 and coeffEq3)
float determinantMatrix33 (float* coeffEq1, float* coeffEq2, float* coeffEq3)
{
	float determinant;
	
	determinant=coeffEq1[0]*(coeffEq2[1]*coeffEq3[2] - coeffEq3[1]*coeffEq2[2]);
	determinant-=coeffEq1[1]*(coeffEq2[0]*coeffEq3[2] - coeffEq2[2]*coeffEq3[0]);
	determinant+=coeffEq1[2]*(coeffEq2[0]*coeffEq3[1] - coeffEq2[1]*coeffEq3[0]);

	return(determinant);
}

void updateIndexPivot(int i, int* index)
{
	switch(i)
	{
		case 0: index[0]=1;index[1]=2;
			break;
		case 1: index[0]=0;index[1]=2;
			break;
		case 2: index[0]=0;index[1]=1;
			break;
		default: fprintf(stderr,"Error updateIndex() i must belong to (0,1,2) and is = %d\n",i);
			exit(4);
	}
}

int verify3Equations(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result)
{
	float sum;

	sum=coeffEq1[0]*result[0]+coeffEq1[1]*result[1]+coeffEq1[2]*result[2];
	if(sum>1.0+EPSILON) return(0);
	sum=coeffEq2[0]*result[0]+coeffEq2[1]*result[1]+coeffEq2[2]*result[2];
	if(sum>1.0+EPSILON) return(0);
	sum=coeffEq3[0]*result[0]+coeffEq3[1]*result[1]+coeffEq3[2]*result[2];
	if(sum>1.0+EPSILON) return(0);

	return(1);
}

float solveCase3EquationsDependant(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result)
{
	int i, eqNumber=1, index;
	float ratio1, ratio2, ratio3, coeff[3], min;

	for(i=0;i<3;i++) if(coeffEq1[i]<EPSILON) return(-1.0);
	for(i=0;i<3;i++) if(coeffEq2[i]<EPSILON) return(-1.0);
	for(i=0;i<3;i++) if(coeffEq3[i]<EPSILON) return(-1.0);

	ratio1=coeffEq1[0]/coeffEq2[0];
	ratio2=coeffEq1[0]/coeffEq3[0];
	ratio3=coeffEq2[0]/coeffEq3[0];

	for(i=1;i<3;i++) if ( (coeffEq1[i]/coeffEq2[i] > ratio1 + EPSILON)  || (coeffEq1[i]/coeffEq2[i] < ratio1 - EPSILON) ) return(-1);
	for(i=1;i<3;i++) if ( (coeffEq1[i]/coeffEq3[i] > ratio2 + EPSILON)  || (coeffEq1[i]/coeffEq3[i] < ratio2 - EPSILON) ) return(-1);
	for(i=1;i<3;i++) if ( (coeffEq2[i]/coeffEq3[i] > ratio3 + EPSILON)  || (coeffEq2[i]/coeffEq3[i] < ratio3 - EPSILON) ) return(-1);

	//The three equations are the same (a un facteur pres)

	//We take the greatest equation 
	eqNumber=0;
	if(coeffEq1[0]>coeffEq2[0] && coeffEq1[0]>coeffEq3[0]) eqNumber=1;
	if(coeffEq2[0]>coeffEq1[0] && coeffEq2[0]>coeffEq3[0]) eqNumber=2;
	if(coeffEq3[0]>coeffEq1[0] && coeffEq3[0]>coeffEq2[0]) eqNumber=3;

	if(eqNumber==1){ coeff[0]=coeffEq1[0]; coeff[1]=coeffEq1[1];coeff[2]=coeffEq1[2];}
	if(eqNumber==2){ coeff[0]=coeffEq2[0]; coeff[1]=coeffEq2[1];coeff[2]=coeffEq2[2];}
	if(eqNumber==3){ coeff[0]=coeffEq3[0]; coeff[1]=coeffEq3[1];coeff[2]=coeffEq3[2];}

	//The three coeffs are the same : we share the bandwitdh equally between the three 
	if( (fabs(coeff[0]-coeff[1]) > EPSILON) && (fabs(coeff[1]-coeff[2]) > EPSILON) )
	{
		result[0]=1.0/(3.0*coeff[0]);
		result[1]=1.0/(3.0*coeff[0]);
		result[2]=1.0/(3.0*coeff[0]);
		return(1.0/coeff[0]);//the sum
	}

	
	//Two coeffs are the same : we share the bandwitdh equally between the two
	min=coeff[0]; index=0;
	if(coeff[1]<min) 
	{
		min=coeff[1]; index=1;
	}
	if(coeff[2]<min) 
	{
		min=coeff[2]; index=2;
	}
	
	result[0]=0.0;
	result[1]=0.0;
	result[2]=0.0;
	for(i=0;i<3;i++)
	{
		//The min is hared with another coeff: they share the bandwitdh (the other one is 0)
		if( (i!=index) && fabs(min-coeff[i])<EPSILON)
		{
			result[i]=1.0/(2.0*coeff[i]);
			result[index]=1.0/(2.0*coeff[i]);
			return(1.0/coeff[0]);//the sum
		}
	}


	//There is only one minimum: it takes all bandwidth
	if(coeff[index]>0.0){ result[index]=1.0/coeff[index]; return(result[index]);}
	return(0.0);

}



float degenerate3Equations(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result)
{
	int i, j, sumD=0.0,sumTmp, indexi[2], indexj[2] ;
	float matrix[3][3], resultTmp[3], resultTmp2[3];


	memcpy(matrix[0],coeffEq1,3*sizeof(float));
	memcpy(matrix[1],coeffEq2,3*sizeof(float));
	memcpy(matrix[2],coeffEq3,3*sizeof(float));

	//Particular case the three equations are all the same 
	if((sumD=solveCase3EquationsDependant(coeffEq1, coeffEq2, coeffEq3, result))>0.0) return(sumD);
	

	//Cas degenere -> on s'appuie sur la gestion du cas degenere a 2 equations pour renvoyer une solution
	//We try all combinations of two equations: (i,j) is the "pivot"
	for(i=0;i<3;i++)
	{
		updateIndexPivot(i,indexi);
		for(j=0;j<3;j++)
		{
			updateIndexPivot(j,indexj);

			//We may ave a submatrix 2x2 that is null
			if( (matrix[indexi[0]][indexj[0]]<EPSILON) && (matrix[indexi[0]][indexj[1]]<EPSILON) &&  (matrix[indexi[1]][indexj[0]]<EPSILON)  && (matrix[indexi[1]][indexj[1]]<EPSILON)) continue; 
		
			sumTmp=solve2equations(matrix[indexi[0]], matrix[indexi[1]], indexj[0], indexj[1], resultTmp2);
			resultTmp[indexj[0]]=resultTmp2[0];
			resultTmp[indexj[1]]=resultTmp2[1];
			resultTmp[j]=0.0;

			if( sumTmp>0.0 && verify3Equations(coeffEq1, coeffEq2, coeffEq3,resultTmp) && sumD<sumTmp)//Solution that verifies all constraints and is better
			{
				sumD=sumTmp;
				result[0]=resultTmp[0];
				result[1]=resultTmp[1];
				result[2]=resultTmp[2];
			}

		}
	}

	return(result[0]+result[1]+result[3]);
}

/*
* coeffEq1=[coeff1, coeff2, coeff3] for equation: coeff1*x+coeff2*y+coeff3*z=1 (idem coeffEq2 and coeffEq3)
* result will host the results (x,y)
* Return the sum x+y if there is a solution ; return -1.0 otherwise. 
*/
float solve3equations(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result)
{
	float determinant, vector1[3], vector2[3], vector3[3], vectorb[3],b=1.0;

	determinant=determinantMatrix33 (coeffEq1,coeffEq2,coeffEq3);
	if( (determinant==0.0) || (fabsf(determinant)<EPSILON) ) return(degenerate3Equations(coeffEq1, coeffEq2, coeffEq3, result));


	//Vector update for the computations of the determinant: http://calculis.net/systeme-equation-3-inconnues
	//They correspond to the "transpose" of the coeff, we'll chnage one of them and compute the determinant
	vector1[0]=coeffEq1[0]; vector1[1]=coeffEq2[0]; vector1[2]=coeffEq3[0];
	vector2[0]=coeffEq1[1]; vector2[1]=coeffEq2[1]; vector2[2]=coeffEq3[1];
	vector3[0]=coeffEq1[2]; vector3[1]=coeffEq2[2]; vector3[2]=coeffEq3[2];
	vectorb[0]=b; vector3[1]=b; vector3[2]=b;

	result[0]=determinantMatrix33 (vectorb,vector2,vector3)/determinant;
	result[1]=determinantMatrix33 (vector1,vectorb,vector3)/determinant;
	result[2]=determinantMatrix33 (vector1,vector2,vectorb)/determinant;

	
	if(result[0]<0.0) result[0]=0.0;
	if(result[1]<0.0) result[1]=0.0;
	if(result[2]<0.0) result[2]=0.0;

	return(result[0]+result[1]+result[2]);
}//end of fct solve2equations

float solveOneValidPath(int nbOfRoutes, float* result)
{
	int pathNumber, i;
	float max=0.0;//The benefit function: sum of the Di on each valid path
	
	result[0]=0.0; result[1]=0.0;result[2]=0.0;
	for(pathNumber=0;pathNumber<nbOfRoutes;pathNumber++)//Search for the valid path
	{
		//DEBUG: j'ai retire le if ci-dessous: pas genant a priori...mais peut etre plus long a execute (conditionne par algo?)
		//if(coeffList[0][pathNumber]>0)//Valid path, the first index corresponds to node 0
		//{
			for(i=0;i<nbOfValidLinks;i++)//compute the max of the coeff on this path
			{
				if(max<coeffList[i][pathNumber]) max=coeffList[i][pathNumber];
				if(DEBUG) fprintf(stderr,"coeff[%d][%d]=%f\n",i,pathNumber,coeffList[i][pathNumber]);
				//if(coeffList[i][pathNumber]<=0.0) fprintf(stderr,"Error i=%d coeff[%d][%d]=%f\n",i,i,pathNumber,coeffList[i][pathNumber]);
			}
			result[pathNumber]=1.0/max;
		//} 
	}

	//The maximum bandwidth is the inverse
	return(1.0/max);
}


//On verifie que la solution trouve verifie bien l'ensemble des contraintes sur l'utilisation
//Cette verification se fait sur tous les liens valides. 
int verifyAllEquations(int nbOfValidRoutes, int* indexRoutes, float* result)
{
	int i, j;
	float sum;


	for(i=0;i<nbOfValidLinks;i++)
	{
		sum=0.0;
		for(j=0;j<nbOfValidRoutes;j++) sum+=coeffList[i][indexRoutes[j]]*result[j];

		if(sum>1.0+EPSILON)
		{
			return(0);
		}
	}
	
	return(1);
}

float solveTwoValidPaths(int nbOfRoutes, float* result)
{
	int pathNumber, i=0, j, indexPath[2];
	float sumTmp, resultTmp[3],  sumD=0.0;//The benefit function: sum of the Di on each valid path
	
	//DEBUG: A modifier -- fonction de algo???? - par contre mettre cette condition en dessous pour ne parcourir que les liens avec coeff > 0.0
	for(pathNumber=0;pathNumber<nbOfRoutes;pathNumber++)//Search for the valid path
	{
		//if(nodes[0].routingTable[pathNumber].coeff[pathNumber]>0.0){ indexPath[i]=pathNumber; i++;}
		indexPath[i]=pathNumber; i++;	//Point a surveiller
	}

	for(i=0;i<nbOfValidLinks-1;i++)
	{
		for(j=i+1;j<nbOfValidLinks;j++)
		{
			sumTmp=solve2equations(coeffList[i], coeffList[j], indexPath[0], indexPath[1], resultTmp);

			if( sumTmp>0.0 && verifyAllEquations(2,indexPath, resultTmp) && sumD<sumTmp)//Solution that verifies all constraints and is better
			{
				sumD=sumTmp;
				//C'est ici que l'on met les debits sur les bons chemins - dans les appels qui suivent uniquement les deux premieres cases de result sont mis a jour 
				//independemment du bon numero de chemin. 
				result[0]=0.0; result[1]=0.0; result[2]=0.0;
				result[indexPath[0]]=resultTmp[0];
				result[indexPath[1]]=resultTmp[1];
			}
		}
	}

	return(sumD);
}

//nbOfRoutes is supposed to be three here -> and we computed only three routes so that routes 0, 1 and 2 (no need to index as for solveTwoValidPaths)
float solveThreeValidPaths(int nbOfRoutes, float* result)
{
	int i, j, k, indexPath[3];
	float sumTmp, resultTmp[3],  sumD=0.0;//The benefit function: sum of the Di on each valid path
	
	indexPath[0]=0; indexPath[1]=1; indexPath[2]=2; 
	resultTmp[0]=0.0; resultTmp[1]=0.0; resultTmp[2]=0.0;

	for(i=0;i<nbOfValidLinks-2;i++)
	{
		for(j=i+1;j<nbOfValidLinks-1;j++)
		{
			for(k=j+1;k<nbOfValidLinks;k++)
			{
				sumTmp=solve3equations(coeffList[i], coeffList[j], coeffList[k], resultTmp);
					
				if( sumTmp>0.0 && verifyAllEquations(3,indexPath, resultTmp) && sumD<sumTmp)//Solution that verifies all constraints and is better
				{
					sumD=sumTmp;
					result[0]=resultTmp[0];
					result[1]=resultTmp[1];
					result[2]=resultTmp[2];
				}
			}
		}
	}

	return(sumD);
}


//algo is 0 if we look for an optimal - 1 otherwise (heuristic) 
float estimateThroughput(int nbOfRoutes, float* result, int algo)
{
	int nbOfValidPaths;
	float sumD=0.0;//The benefit function: sum of the Di on each valid path
	
	nbOfValidPaths=setCoefficientAllNodes(nbOfRoutes, algo);//Set the coefficient

	if(nbOfValidPaths==0) 
	{
		/*
		fprintf(stderr,"Error fct estimateThroughput(): no route to destination.\n");
		exit(4);
		*/
		return(0.0);
	}

	if(nbOfValidPaths==1) sumD=solveOneValidPath(nbOfRoutes,result);
	if(nbOfValidPaths==2) sumD=solveTwoValidPaths(nbOfRoutes,result);
	if(nbOfValidPaths==3) sumD=solveThreeValidPaths(nbOfRoutes,result);

	return(sumD);
}


