/*
 *
 *      Author: Anthony
 */


#ifndef COST_H_
#define COST_H_

#include "heuristic.h"
#include "metric.h"

//Check if the route routeNumber is active between id1 and id2
//i.e. if the routeNumberth entry of the routing table of node id1 is not empty and has id2 as next hop.
//Moreover, check if freq is used between these two nodes.
//Return 1 if yes, 0 otherwise.
int routeActiveOnFreq(int id1, int id2, int freq, int routeNumber);

//If id1 and id2 use freq, update the coeffArray for each route with id2 as next hop. 
//Update means add 1/getMetric() to the current coeff. 
void updateCoefficientArray(float* coeffArray, int id1, int id2, int freq, int nbOfRoutes);

//Compute the coefficient of the inequality (for the inequality linear systems)
//Must be done for each on the paths.
//These coefficients are added to an array associate to the corresponding routing entry on node id1.
//So we assume that there is a path/route to destination between id1 and id2 
//The requency and radio MUST be upadted
void setCoefficientLink(int id1, int id2, int pathNumber, int freq, int nbOfRoutes);

//Set all coefficient at 0 from node start on pathNumber
void setCoefficientNull(int start, int pathNumber);

//algo=0 if we search for the optimal
//algo=1 if we use the heuristic
//The difference is that for the optimal all the link of the path are 0 if there is a link that is not valid (no common frequnecy between neighbor)
//whereas we set the coefficient until the no valid link in th ecase of the heuristic
//Return the number of valide paths even partial (set the size of the linear system to solve).
int setCoefficientAllNodes(int nbOfRoutes, int algo);


float solveCase3EquationsDependant(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result);
void updateIndexPivot(int i, int* index);
int verify3Equations(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result);
float degenerate3Equations(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result);


/*
* coeffEq1=[coeff1, coeff2] for equation: coef1*x+coef2*y=1 (idem coeffEq2)
* result will host the results (x,y)
* Return the sum x+y if there is a solution ; return -1.0 otherwise. 
* index1, index2 are the coefficient to take into account (it is not always 0 and 1 / it may be 0 and 2) ; it depends on the valid routes
*/
float solve2equations(float* coeffEq1, float* coeffEq2, int index1, int index2, float* result);

//coeffEq1=[coeff1, coeff2, coeff3] for equation: coeff1*x+coeff2*y+coeff3*z=1 (idem coeffEq2 and coeffEq3)
float determinantMatrix33 (float* coeffEq1, float* coeffEq2, float* coeffEq3);

/*
* coeffEq1=[coeff1, coeff2, coeff3] for equation: coeff1*x+coeff2*y+coeff3*z=1 (idem coeffEq2 and coeffEq3)
* result will host the results (x,y)
* Return the sum x+y if there is a solution ; return -1.0 otherwise. 
*/
float solve3equations(float* coeffEq1, float* coeffEq2, float* coeffEq3, float* result);

int verifyAllEquations(int nbOfRoutes, int* indexRoutes, float* result);

float solveOneValidPath(int nbOfRoutes, float* result);
float solveTwoValidPaths(int nbOfRoutes, float* result);
float estimateThroughput(int nbOfRoutes, float* result, int algo);







#endif


