#include "init.h"

#define ROUTES_CONFLICT	1	/*Pour pathRand avec 2 et 3 routes - indique si les routes sont en conflits
				//				0	ou 	0
				//			       / \ 	       / \
				//			      1---2	       1  2
				//			      Cas 1	      Cas 0
				*/

int nbPerLine;//use for the grid topology (indicates the number of nodes per line).
float edgeLength;//use for the grid topology (indicates the length of a link).

void error(char* string)
{
	perror(string);
	exit(-1);
}//end of function error()


void pathTopology(int variant, int nbOfRoutes)
{
	int i;
	float random, h, vert, l;//h horizontal distance - vert vertical distance - l: randomness added to the vertical location

	if(variant==0) random=0.0; else random=1.0;

	if( (nbOfRoutes!=1) && (nbOfRoutes!=2) && (nbOfRoutes!=3)) 
	{
		fprintf(stderr,"Error init.c pathTopology(): for this topology the number of routes must 1, 2 or 3.\n");
		exit(1);
	}
	
	if( (nbOfRoutes==2) && ( (nbOfNodes%2) ==1)) 
	{
		fprintf(stderr,"Error init.c pathTopology(): for this topology and for 2 routes the number of nodes must be even.\n");
		exit(1);
	}
	
	if( (nbOfRoutes==3) && ( ((nbOfNodes-2)%3)!=0)) 
	{
		fprintf(stderr,"Error init.c pathTopology(): for this topology and for 3 routes the number of nodes -2 must be a multiple of 3.\n");
		exit(1);
	}

	if(nbOfNodes<4) 
	{
		fprintf(stderr,"Error init.c pathTopology(): for this topology, the minimum number of nodes is 4.\n");
		exit(1);
	}

	if(nbOfRoutes==1) 
	{
		//Source
		nodes[0].x=0.0;
		nodes[0].y=0.0;
	
		//La distance au noeud precedent est en moyenne de 3*R/5 (cas deterministe ou aleatoire)
		for(i=1;i<nbOfNodes;i++)
		{
			l=2.0*R*(2.0*rand()/(1.0*RAND_MAX) - 1.0)/5.0;//distribue uniformement dans [-2R/5 ; 2R/5]
			nodes[i].x=nodes[i-1].x + 3.0*R/5.0+random*l;
			nodes[i].y=0.0;
		}
	}

	if(nbOfRoutes==2) 
	{
		//h=sqrt(15.0)*R/4.0;//Pas liens diagonaux
		h=2.0*R/3.0;//avec liens diagonaux
		//vert=(1.1*R)/2.0;
		vert=(0.9*R)/2.0;
	
		if(ROUTES_CONFLICT==0) h=R*1.2;

		//Destination
		nodes[nbOfNodes-1].x=h/2.0;
		nodes[nbOfNodes-1].y=0.0;

		for(i=nbOfNodes-2;i>1;i-=2)
		{
			l=R/8.0 - R*(1.0*rand())/(4.0*RAND_MAX);//Randomness of R/4.0
			nodes[i].x=h;
			nodes[i].y=vert+random*l;

			l=R/8.0 - R*(1.0*rand())/(4.0*RAND_MAX);//Randomness of R/4.0
			nodes[i-1].x=0.0;
			nodes[i-1].y=vert+random*l;

			//vert+=3.0*R/4.0; //Pas de liens diagonaux 
			//vert+=2.0*R/3.0; //Liens diagonaux 
			vert+=1.1*R/2.0; //Liens diagonaux 
		}
		
		//Source
		//vert=vert - 3.0*R/4.0 + R/2.0;//We need to remove the increment that has been set at the end of the for loop - and add the good valeue (R/2.0)
		vert=vert - 2.0*R/3.0 + R/2.0;//We need to remove the increment that has been set at the end of the for loop - and add the good valeue (R/2.0)
		nodes[0].x=h/2.0;
		nodes[0].y=vert;

	}

	if(nbOfRoutes==3) 
	{
		h=3.0*R/4.0;
		vert=1.1*R/2.0;
	
		//Destination
		nodes[nbOfNodes-1].x=h;
		nodes[nbOfNodes-1].y=0.0;

		for(i=nbOfNodes-2;i>2;i-=3)
		{
			fprintf(stderr,"DEBUG- indice = %d %d %d\n",i,i-1,i-2);
			l=R/8.0 - R*(1.0*rand())/(4.0*RAND_MAX);//Randomness of R/4.0
			nodes[i].x=2.0*h;
			nodes[i].y=vert+random*l;

			l=R/8.0 - R*(1.0*rand())/(4.0*RAND_MAX);//Randomness of R/4.0
			nodes[i-1].x=h;
			nodes[i-1].y=vert+random*l;

			l=R/8.0 - R*(1.0*rand())/(4.0*RAND_MAX);//Randomness of R/4.0
			nodes[i-2].x=0.0;
			nodes[i-2].y=vert+random*l;

			vert+=3.0*R/4.0;
		}
		
		//Source
		vert=vert - 3.0*R/4.0 +R/2.0;//We need to remove the increment that has been set at the end of the for loop - and add the good valeue (R/2.0)
		nodes[0].x=h;
		nodes[0].y=vert;

	}

	for(i=0;i<nbOfNodes;i++)
		fprintf(stderr,"Node %d x=%f y=%f\n",i,nodes[i].x,nodes[i].y);

}//end of fct pathTopology()

//Initialize the different entries of the node "id".
void initNode(int id, int variant)
{
	int i;

	nodes[id].id=id;
	nodes[id].nbOfRadios=nbOfRadios;
	nodes[id].nbN1=-1;
	nodes[id].nbN2=-1;
	nodes[id].routeFlag=0;

	//Assign frequency -1 to all radios
	for(i=0;i<MAX_FREQ;i++){
		nodes[id].frequency[i]=-1;
	}

    	//Set the neighorhood and the 2-neighborhood with -1 (it means not assigned).
	for(i=0;i<MAX_N1;i++){
		nodes[id].N1[i].id=-1;
		nodes[id].N1[i].nbOfRadios=nbOfRadios;
		nodes[id].N1[i].frequency=-1;
		nodes[id].N1[i].metric=-1.0;
	}

	for(i=0;i<MAX_N2;i++){
		nodes[id].N2[i].id=-1;
		nodes[id].N2[i].nbOfRadios=nbOfRadios;
		nodes[id].N2[i].frequency=-1;
		nodes[id].N1[i].metric=-1.0;
	}

	//initializes the routing table
	initRoutingTable(id);

}//end of function initNode()

//Malloc the ressource for the routing table and initialize it (with -1)
struct routingEntry* initRoutingTable(int id)
{
	int i;
	struct routingEntry* ptrRoutingTable;

	if((ptrRoutingTable=(struct routingEntry *) malloc(MAX_NBOFROUTES*sizeof(struct routingEntry)))==NULL)
		error("Error: initRoutingTable() -> malloc");

	nodes[id].routingTable=ptrRoutingTable;
	//printf("--MALLOC-- Adresse routingTable %d: %x\n",id,nodes[id].routingTable);
	for(i=0;i<MAX_NBOFROUTES;i++)
	{
		ptrRoutingTable[i].destination=-1;
		ptrRoutingTable[i].nextHop=-1;
	}

	return(ptrRoutingTable);
}//end of function initRoutingTable()

//Free the ressource (the two mallocs) for node id
void freeNodes()
{
	int i;

	for(i=0;i<nbOfNodes;i++){
		//printf("--FREE-- Adresse routingTable %d: %x\n",i,nodes[i].routingTable);
		free(nodes[i].routingTable);
	}
	//printf("--FREE-- Nodes: %x\n",nodes);
	//free(nodes);//Done in the main
}

//indicates if the node id1 is in the neighborhood of node id2
//return 1 if yes and 0 otherwise
int isNeighbor(int id1, int id2)
{
    if (((pow((nodes[id1].x-nodes[id2].x),2) + pow((nodes[id1].y-nodes[id2].y),2))<=pow(R,2))&&(id1!=id2)) return 1;
    else return 0;
}

//indicates if the node id1 is in the 2-neighborhood of node id2
//return 1 if yes and 0 otherwise
int is2Neighbor(int id1, int id2)
{
    int i,k;
    //If id1=id2 or they are neighbor, then they are NOT 2-hop neighbor (return 0)
    if ((id1==id2)||(isNeighbor(id1,id2))) return 0;
    for (i=0;i<nodes[id2].nbN1;i++)
    {
    	k=nodes[id2].N1[i].id;
   	//If we find a common neighbor, they are 2-hop neighbor, return 1
	if (isNeighbor(id1,k)) return 1;
    }
    return 0;
}

float getMetric(int id1, int id2)
{
	double distance= sqrt( pow((nodes[id1].x-nodes[id2].x),2.0) + pow((nodes[id1].y-nodes[id2].y),2.0) );

	if(distance < 0.0)
	{
		fprintf(stderr,"Error in fct getMetric(): distance between two nodes is less than 0.0 (=%e) - abort...\n",distance);
		exit(2);
	}

	// Mis a jour en fonction IdealWifiManager (voir le fichier rounde_idealwifimanager.ods)
	// 
	if(distance <= 1.0) return (23.688);
	if(distance <= 4.0) return (23.686);
	if(distance <= 7.0) return (23.685);
	if(distance <= 10.0) return (23.684);
	if(distance <= 13.0) return (23.682);
	if(distance <= 16.0) return (23.681);
	if(distance <= 19.0) return (23.679);
	if(distance <= 22.0) return (23.678);
	if(distance <= 25.0) return (23.392);
	if(distance <= 28.0) return (21.821);
	if(distance <= 31.0) return (19.227);
	if(distance <= 34.0) return (19.226);
	if(distance <= 37.0) return (19.217);
	if(distance <= 40.0) return (18.939);
	if(distance <= 43.0) return (14.997);
	if(distance <= 46.0) return (14.995);
	if(distance <= 49.0) return (14.936);
	if(distance <= 52.0) return (14.079);
	if(distance <= 55.0) return (12.218);
	if(distance <= 58.0) return (12.217);
	if(distance <= 61.0) return (12.215);
	if(distance <= 64.0) return (12.201);
	if(distance <= 67.0) return (12.069);
	if(distance <= 70.0) return (11.174);
	if(distance <= 73.0) return (8.994);
	if(distance <= 76.0) return (8.993);
	if(distance <= 79.0) return (8.989);
	if(distance <= 82.0) return (8.951);
	if(distance <= 85.0) return (8.713);
	if(distance <= 88.0) return (7.557);
	if(distance <= 91.0) return (4.997);
	if(distance <= 94.0) return (4.997);
	if(distance <= 97.0) return (4.995);
	if(distance <= 100.0) return (4.992);
	if(distance <= 103.0) return (4.972);
	if(distance <= 106.0) return (4.903);
	if(distance <= 109.0) return (4.665);
	if(distance <= 112.0) return (3.855);
	if(distance <= 115.0) return (1.917);
	if(distance <= 118.0) return (.244);
	if(distance <= 121.0) return (.001);
	if(distance <= 124.0) return (0.0);


	return(0.0);
}


//Update the neighborhood for node id (on the common channel)
int updateN1(int id)
{
    int i,j=0;
    for(i=0;i<nbOfNodes;i++)
    {
            if (isNeighbor(i,id))
            {
	    	//Add i to the list of neighbors
                nodes[id].N1[j].id=i;
		nodes[id].N1[j].nbOfRadios=nbOfRadios;
		nodes[id].N1[j].frequency=-1;//Not allocated
		nodes[id].N1[j].radio=-1;//Not allocated
		nodes[id].N1[j].metric=getMetric(id,i);
                j++;
            }
	    if(j>=MAX_N1){
		fprintf(stderr,"Error updateN1 (init.c): the value of the constant MAX_N1 is not sufficently large\n");
		exit(-1);
	    }
    }//end of for

    nodes[id].nbN1=j;
    nodes[id].frequency[0]=0;

    return(1);
}//end of function updateN1()

//Update the 2-neighborhood for node id (on the common channel)
int updateN2(int id)
{
	int i,j=0;
	for(i=0;i<nbOfNodes;i++)
	{
		if (is2Neighbor(i,id))
		{
			//Add i to the list of 2-neighbors
			nodes[id].N2[j].id=i;
			nodes[id].N2[j].nbOfRadios=nbOfRadios;
			nodes[id].N2[j].frequency=0;
			j++;
       		}
        	if(j>=MAX_N2){
         		fprintf(stderr,"Error updateN2 (init.c): the value of the constant MAX_N2 is not sufficiently large\n");
			exit(-1);
	  	}
	}
	nodes[id].nbN2=j;
	nodes[id].frequency[0]=0;

	return(1);
}//end of function updateN2()

//Initialize the array nodes (id, locations, N1 and N2) and allocate the memory for the routing table.
//Frequency are not assigned and the routing table is not compute.
void initArrayNodes(int variant, int nbOfRoutes)
{
	int i;

	if((variant<0)||(variant>1)){
		fprintf(stderr,"Bad choice of topology: it must be 0 or 1.\n");
		exit(1);
	}

	//We set the topology first
	pathTopology(variant,nbOfRoutes);

	for(i=0;i<nbOfNodes;i++) initNode(i,variant);
	for(i=0;i<nbOfNodes;i++) updateN1(i);
	for(i=0;i<nbOfNodes;i++) updateN2(i);

	//DEBUG
 	//printArrayNodes();

}//end of function initArrayNode()


//Essaye de calculer nbOfRoutes routes 
//Les sources destination dependent de la topologie:
//	- si path ou pathRand --> source=0 destination=nbOfNodes-1 et toutes les routes joignent ces deux noeuds
//	- si rand ou grid --> les couples (source , destination) sont tires au hasard et il y a une seule route entre elle NON VOIR CI-DESSOUS
int computeRoutes(int variantTopology, int nbOfRoutes)
{
	int id=0, nbOfComputedRoutes=0; 

	if( (variantTopology==1) || (variantTopology ==2) || (variantTopology==3))
	{
		while( (nbOfComputedRoutes < nbOfRoutes) && (updateRouteSourceDestination(0, nbOfNodes-1, nbOfComputedRoutes)>=0) )
		{
			sourceArray[nbOfComputedRoutes]=0; destinationArray[nbOfComputedRoutes]=nbOfNodes-1; 
			nbOfComputedRoutes++;
		}

		return(nbOfComputedRoutes);
	}

	//Topologie random: pas besoin de tirer au hasard (puisuqe les noeuds sont deja distribue au hasard)
	//Source=0 (puis 2, 4, etc.) destination=1 (puis 3, 5, etc.)
	if( variantTopology == 0 )
	{
		if( 2*nbOfRoutes > nbOfNodes)
		{
			fprintf(stderr,"Error in computeRoutes(): the number of nodes is not sufficient for this number of routes.\n"); 
			exit(1);
		}

		while( (nbOfComputedRoutes < nbOfRoutes) && (id<nbOfNodes-1) )
		{
			if( updateRouteSourceDestination(0, nbOfNodes-1, nbOfComputedRoutes)>=0 )
			{ 
				sourceArray[nbOfComputedRoutes]=id; destinationArray[nbOfComputedRoutes]=id+1; 
				nbOfComputedRoutes++; 
			}
			id+=2;
		}

		return(nbOfComputedRoutes);
	}


	/*
	if(variantTopology==1)
	{
		fprintf(stderr,"Error in computeRoutes(): this topology is not taken into account (must be coded).\n");
		exit(1);
	}
	*/

	return(0);
}

//Print the fields of the nodes in the array nodes
void printArrayNodes()
{
	int i,j,index;

	for(i=0;i<nbOfNodes;i++)
	{
		printf("---------- Node %d -----------\n",i);
		printf("-- x=%f y=%f\n",nodes[i].x,nodes[i].y);
		printf("-- nbOfRadios=%d nbN1=%d nbN2=%d Frequency=",nodes[i].nbOfRadios,nodes[i].nbN1,nodes[i].nbN2);

		for(j=0;j<nodes[i].nbOfRadios;j++) printf("%d ",nodes[i].frequency[j]); 
		printf("\n");

		printf("-- N1 = "); for(j=0;j<nodes[i].nbN1;j++) printf("%d ",nodes[i].N1[j].id); printf("\n");
		printf("-- N2 = "); for(j=0;j<nodes[i].nbN2;j++) printf("%d ",nodes[i].N2[j].id); printf("\n");
		printf("-- Routing Table: dest / nextHop / cost / radio /frequency\n");
		for(j=0;j<nbOfNodes;j++)
		{
			index=getIndexN1(nodes[i],j);

			//nodes[id].N1[index].frequency=bestFreq;
			//nodes[id].N1[index].radio=bestRadioId;
			printf("%d / %d / %f / ",nodes[i].routingTable[j].destination,nodes[i].routingTable[j].nextHop,nodes[i].routingTable[j].cost);
			if(index>=0) printf("%d / %d\n",nodes[i].N1[index].radio,nodes[i].N1[index].frequency);
			else printf("-1 / -1\n");
		}
	}

}//end of function printArrayNodes()

