/*
 ============================================================================
 Name        : main.c
 Author      : Anthony Busson
 Version     :
 ============================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include "type.h"
#include "init.h"
#include <time.h>
#include "metric.h"
#include "ns3.h"

//Global variables
int nbOfNodes;		//Number of nodes
struct node* nodes;	//Array with the nodes
int nbOfRadios;		//Number of radios per node
int maxFreq;		//Number of orthogonal frequencies
int algorithm;		//Algorithm number used to allocate the frequencies
//float	L;		//Length of a side of the observation window (LxL)
float	R;		//Radio scope of the nodes
int ns3Version=2;	//1: ns-3.5 without ECMP
			//2: ns-3.7.1 with ECMP

int disjointPaths; //If 1 there is only one route per node - If 0 we may have several routes using the same node
int directionnels; //If 1 there is conflict directional - If 0 there is conflict bidirectional


void usage(char* string)
{
	fprintf(stderr,"Usage: %s topology nbOfNodes nbOfRadios maxFreq nbOfSamples nbOfRoutes algorithm \n",string);
	fprintf(stderr,"-----------------------------------------------------------------\n");
	fprintf(stderr,"-- topology: 0-> pathTopology 1-> pathTopologyRandom\n");
	fprintf(stderr,"-- nbOfNodes: number of nodes\n");
	fprintf(stderr,"-- nbOfRadios: number of radios\n");
	fprintf(stderr,"-- maxFreq: number of available frequencies\n");
	fprintf(stderr,"-- nbOfSamples: number of NS3 file to simulate the capacity\n");
	fprintf(stderr,"-- nbOfRoutes: number of routes between the source and the destination\n");
	fprintf(stderr,"-- algorithm: One frequency=0 Minimize the conflict graph (optimal)=1  Throughput_Heuristic=2 Throughput_Optimal=3\n");
	fprintf(stderr,"-----------------------------------------------------------------\n");
	exit(-1);
}

int main(int argc, char* argv[])
{

	int nbOfSamples,i,topology, nbOfRoutes;
	char fileName[255];

	if((ns3Version!=1)&&(ns3Version!=2)){
		fprintf(stderr,"La version de ns3 est incorrect (=%d et doit etre 1 ou 2)\n",ns3Version);
		exit(15);
	}


	if(argc < 8) usage(argv[0]);
	topology=atoi(argv[1]);
	nbOfNodes=atoi(argv[2]);
	nbOfRadios=atoi(argv[3]);
	maxFreq=atoi(argv[4]);
	R=100; 					//Should keep this value if simulated with ns-3.
	nbOfSamples=atoi(argv[5]);
	nbOfRoutes=atoi(argv[6]);
	algorithm=atoi(argv[7]);
	disjointPaths = 1;
	directionnels = 0;

	fprintf(stderr,"--DEBUG-- topology = %d\n",topology);
	fprintf(stderr,"--DEBUG-- nbOfNodes = %d\n",nbOfNodes);
	fprintf(stderr,"--DEBUG-- nbOfRadios = %d\n",nbOfRadios);
	fprintf(stderr,"--DEBUG-- maxFreq = %d\n",maxFreq);
	fprintf(stderr,"--DEBUG-- Radio range=%f\n",R);
	fprintf(stderr,"--DEBUG-- nbOfSamples = %d\n",nbOfSamples);
	fprintf(stderr,"--DEBUG-- nbOfRoutes = %d\n",nbOfRoutes);
	fprintf(stderr,"--DEBUG-- algo = %d\n",algorithm);
	fprintf(stderr,"--DEBUG-- disjointPaths = %d\n",disjointPaths);
	fprintf(stderr,"--DEBUG-- directionnel = %d\n",directionnels);


	time_t seconds=time(NULL);	//A different seed for each computation/simulation.
	//int seconds=1; //To reproduce the same computations/simulations
	srand(seconds);
	//fprintf(stderr,"--DEBUG-- srand=%d\n",seconds);


	//Malloc for the array nodes
	if((nodes=(struct node *) malloc(nbOfNodes*sizeof(struct node)))==NULL) error("Error: main() -> malloc nodes");

	i=1;
	while(i<=nbOfSamples){
		//Name of the NS-3 file
		if(topology==0)
		  sprintf(fileName,"fichiersNS3/pathTopAlgo%dNbradio%dNbFreq%dnbNoeuds%dnbOfRoutes%d-%d.cc",algorithm,nbOfRadios,maxFreq,nbOfNodes,nbOfRoutes,i);
		if(topology==1)
		  sprintf(fileName,"fichiersNS3/pathRandTopAlgo%dNbradio%dNbFreq%dnbNoeuds%dnbOfRoutes%d-%d.cc",algorithm,nbOfRadios,maxFreq,nbOfNodes,nbOfRoutes,i);

		fprintf(stderr,"Nom du fichier = %s\n",fileName);
		fprintf(stderr,"Generation du fichier %d sur %d\n",i,nbOfSamples);
		if(NS3file(topology,nbOfRoutes,fileName)==1) i++;
	}//fin du while

	free(nodes);

	return(0);
}
