/*
 * throughputOptimal.c
 *
 *      Author: Anthony
 */

#include"throughputOptimal.h"


#define DEBUGRECURSIVE 0

float maxCapacite = 0.0; // meilleur capacité

struct node* optimalAlloc;

/* Return 1 if the path is valid, i.e. if there is a radio/freq allocated on the path between src and dest	*/
/* returns 0 otherwise.												*/
int validPath(int pathNumber)
{
	int source, destination, next;

	source=sourceArray[pathNumber];
	destination=destinationArray[pathNumber];
	next = nodes[source].routingTable[pathNumber].nextHop;

	//Parcour du chemin
	while(next != destination)
	{
		if(linkExist2(source,next)==0) return(0);

		source = next;
		next = nodes[source].routingTable[pathNumber].nextHop;
	}

	return(1);
}


//Compute the nextID: the id of the node such that > nodeID and has a valid route
//Return nbOfNodes if there is no such node. 
int computeNextID(int nodeID, int nbOfRoutes)
{
	int i, nextID;

	//We compute the next NodeID (node with an id > nodeID and that is on a path i.e. has a route)
	nextID=nodeID+1;
	while(nextID<nbOfNodes)
	{
		//Is there a route ?
		for(i=0;i<nbOfRoutes;i++) if(nodes[nextID].routingTable[i].destination>=0) return(nextID);
		nextID++;
	}

	return(nextID);
}

//Compute the nextID: the id of the node such that > nodeID and has a valid route
//Return nbOfNodes if there is no such node. 
int computeNextIDAllocation(int nodeID, int nbOfRoutes)
{
	int i, nextID;

	nextID=nodeID+1;
	
	//if we are at the destination (nbOfNodes-1), we return nbOfNodes because this node has no route, but it has to be taken into account for the allocation
	if(nextID==nbOfNodes-1) return(nextID);

	//We compute the next NodeID (node with an id > nodeID and that is on a path i.e. has a route)
	while(nextID<nbOfNodes)
	{
		//Is there a route ?
		for(i=0;i<nbOfRoutes;i++) if(nodes[nextID].routingTable[i].destination>=0) return(nextID);
		nextID++;
	}

	return(nextID);
}

//Une fois que les freq ont ete alloues aux radios il y a encore plusieurs possibilites au niveau de l'assignation
//c'est a dire que l'on peut avoir plusieurs freq/cartes communs a deux voisins. Ce choix impactant la fonction de cout
//On est oblige d'etudier tous les choix possibles. 
void updateFreqRadioRecursive(int nodeID, int nbOfRoutes, float* result)
{
	int freq, neighborID, nextID, i;
	float cost=0.0, resultTmp[3];
	static int optNo=1;

	if(nodeID!=nbOfNodes)	
	{
		//Noeud suivant 
		nextID=computeNextID(nodeID, nbOfRoutes);

		//On parcours les routes (donc les voisins)
		for(i=0;i<nbOfRoutes;i++)
		{
			neighborID=nodes[nodeID].routingTable[i].nextHop;
			if(neighborID>=0)
			{
				//On cherche les frequences communes pour ce voisin, on fait l'assignation et on passe au noeud suivant (appel recursif)
				//Si il n'y a pas de freq communes, on ne fait rien

				for(freq=0;freq<maxFreq;freq++)
				{
					if( (usedfreq(nodeID,freq)>=0) && (usedfreq(neighborID,freq)>=0) )
					{
						assignN1radioFreq(nodeID, neighborID, freq); 
						updateFreqRadioRecursive(nextID, nbOfRoutes, result);
					}
				}//end of for sur les freq
			}
		}
	} else {

		//Nous sommes a la fin de la recursion: calcul de la fonction de cout	
		cost=estimateThroughput(nbOfRoutes, resultTmp, 0);//0 mean that we are in the optimal search (not the heuristic)

		//Si meilleure on sauvegarde l'allocation
		if(cost>maxCapacite)
		{
			maxCapacite=cost;
			memcpy(optimalAlloc,nodes,nbOfNodes*sizeof(struct node));

			fprintf(stderr,"New Optimal (%d) = %f resultTmp[0]=%f [1]=%f [2]=%f\n",optNo,maxCapacite,resultTmp[0],resultTmp[1],resultTmp[2]);
			for(i=0; i<nbOfRoutes; i++) result[i]=resultTmp[i]; 
			
			optNo++;
		}

	}
}

void allocateFrequencyRecursive(int nodeID, int nbOfRoutes, int pathNumber, float* result)
{
	int freq, current, next, i, pathN;

	current=nodeID;
	if(current==0 && pathNumber<nbOfRoutes)
	{
		next=nodes[0].routingTable[pathNumber].nextHop;
		if(next==-1){ fprintf(stderr,"Error in allocateFrequencyRecursive()-: no route found\n"); exit(3);}

		for(freq=-1;freq<maxFreq;freq++)
		{
			//Frequency assignation
			nodes[current].frequency[pathNumber] = freq;
			nodes[next].frequency[1] = freq;
			assignN1radioFreq(current, next, freq); 

			//if(DEBUGRECURSIVE) fprintf(stderr,"\t node %d freq %d\n",nodeID,freq);
			 if(pathNumber<nbOfRoutes-1) allocateFrequencyRecursive(current, nbOfRoutes, pathNumber+1, result);//Recursive call
			 else allocateFrequencyRecursive(current+1, nbOfRoutes, nbOfRoutes, result);//Recursive call
		}//end of for(freq=-1...
	}

	if( current>0 && current<nbOfNodes-1)
	{
		next=-1;
		for(i=0;i<nbOfRoutes;i++)
		{	 
			if(nodes[current].routingTable[i].destination==nbOfNodes-1)
			{
				next=nodes[current].routingTable[i].nextHop;
				pathN=i;
			}
		}
		if(next==-1){ fprintf(stderr,"Error in allocateFrequencyRecursive(): no route found\n"); exit(3);}

		for(freq=-1;freq<maxFreq;freq++)
		{
			//Frequency assignation
			if(next==nbOfNodes-1) nodes[next].frequency[pathN] = freq;//Les differentes radio de la destination
			else nodes[next].frequency[1] = freq;

			nodes[current].frequency[0] = freq;
			assignN1radioFreq(current, next, freq); 

			//if(DEBUGRECURSIVE) fprintf(stderr,"\t node %d freq %d\n",nodeID,freq);
			allocateFrequencyRecursive(current+1, nbOfRoutes, nbOfRoutes, result);//Recursive call

		}//end of for(freq=-1...
	}

	//It is the last node 
	if (current==nbOfNodes-1) 
	{
		//Calcul du cout 
		//Attention c'est ici que l'on doit efectuer l'assignation concrete...des freq sur les radio dans le tableau N1
		//Methode recursive sur les differentes possibilites
		updateFreqRadioRecursive(0, nbOfRoutes,result);
	}

}//end of fct allocateFrequencyRecursive

float throughputOptimal(int nbOfRoutes)
{
	int i;
	float result[3];//for the 2nd cost function, will host the optimal rate for each path (the max is 3 routes)
	float cost1=0.0;

	result[0]=0.0; result[1]=0.0; result[2]=0.0;
	maxCapacite=0.0;//Variable globale

	//Copie de sauvegarde hebergeant l'optimal 
	if((optimalAlloc=(struct node *) malloc(nbOfNodes*sizeof(struct node)))==NULL) error("Error: throughputOptimal() -> malloc nodes");

	allocateFrequencyRecursive(0,nbOfRoutes, 0, result);

	//Copy the best in the nodes array
	memcpy(nodes,optimalAlloc,nbOfNodes*sizeof(struct node));

	//We remove route that are not assigned from the routing tables (a cause de NS-3)
	for(i=0; i< nbOfRoutes;i++) if(validPath(i)==0) initialisationChemin(i,nbOfRoutes);

	//DEBUG - Affichage de l'allocation sur la sortie erreur
	printAllocation(nbOfRoutes);

	//We set the rate for each route (used in the NS-3 simulations)
	//The return capacity is always calculMinCapacity (cost2 is computed in NS-3)
	for(i = 0 ; i < nbOfRoutes ; i++)
	{       	// parcourir la liste de route
		if(nodes[0].routingTable[i].destination>=0)
		{
			nodes[0].routingTable[i].cost2 = result[i];
			fprintf(stderr,"Optimal found: D1=%f D2=%f D3=%f\n",result[0],result[1],result[2]);
			//We return the costFunction1
			cost1+=calculMinCapaciteChemin(i);
		}
	}

	free(optimalAlloc);

	return(cost1);
}

void printAllocation(int nbOfRoutes)
{
	int id=0, i;

	while(id<nbOfNodes)
	{
		fprintf(stderr,"-- Allocation node %d : ",id);
		for(i=0;i<nbOfRadios;i++) fprintf(stderr,"radio %d freq %d /",i,nodes[id].frequency[i]);
		fprintf(stderr," --\n");
		//id=computeNextID(id, nbOfRoutes);
		id++;//REMETTRE LE COMMENTAIRE CI-DESSUS
	}
}




